@@ -1,50 +1,70 @@
 
-$(document).ready(function() {
-   var url_string = window.location.href;
-   var url = new URL(url_string);
-   var error = url.searchParams.get('error');
-   if (error) {
-     renderWarningMessageInLoginStatus(fmt_escape_html(error));
-   } else {
-      if (oauth.enabled) {
-        startWithOAuthLogin();
-      } else {
-        startWithLoginPage();
-      }
+$(document).ready(function() {    
+  var url_string = window.location.href;
+  var url = new URL(url_string);
+  var error = url.searchParams.get('error');
+  if (error) {
+    if (oauth.enabled) {
+      renderWarningMessageInLoginStatus(oauth, fmt_escape_html(error));
     }
+  } else {
+    if (oauth.enabled) {
+      startWithOAuthLogin(oauth);
+    } else {
+      startWithLoginPage();
+      }
+  }
 });
 
 function startWithLoginPage() {
   replace_content('outer', format('login', {}));
   start_app_login();
 }
-function startWithOAuthLogin () {
+function removeDuplicates(array){
+  let output = []
+  for(let item of array) {
+    if(!output.includes(item)) {
+      output.push(item)
+    }
+  }
+  return output
+}
+
+
+function startWithOAuthLogin (oauth) {
   store_pref("oauth-return-to", window.location.hash);
 
   if (!oauth.logged_in) {
-    if (oauth.sp_initiated) {
-      get(oauth.readiness_url, 'application/json', function (req) {
-        if (req.status !== 200) {
-          renderWarningMessageInLoginStatus(oauth.authority + ' does not appear to be a running OAuth2.0 instance or may not have a trusted SSL certificate')
-        } else {
-          replace_content('outer', format('login_oauth', {}))
-          start_app_login()
-        }
-      })
-    } else {
-      replace_content('outer', format('login_oauth', {}))
-      start_app_login()
-    }
+    hasAnyResourceServerReady(oauth, (oauth, warnings) => {  render_login_oauth(oauth, warnings); start_app_login(); })
   } else {
     start_app_login()
   }
 }
+function render_login_oauth(oauth, messages) {
+  let formatData = {}
+  formatData.warnings = []
+  formatData.notAuthorized = false
+  formatData.resource_servers = oauth.resource_servers
+  formatData.declared_resource_servers_count = oauth.declared_resource_servers_count
+  formatData.oauth_disable_basic_auth = oauth.oauth_disable_basic_auth
 
-function renderWarningMessageInLoginStatus (message) {
-  replace_content('outer', format('login_oauth', {}))
-  replace_content('login-status', '<p class="warning">' + message + '</p> <button id="loginWindow" onclick="oauth_initiateLogin()">Click here to log in</button>')
+  if (Array.isArray(messages)) {
+    formatData.warnings = messages
+  } else if (typeof messages == "string") {
+    formatData.warnings = [messages]
+    formatData.notAuthorized = messages == "Not authorized"
+  }
+  replace_content('outer', format('login_oauth', formatData))
+
+  setup_visibility()
+  $('#login').off('click', 'div.section h2, div.section-hidden h2');
+  $('#login').on('click', 'div.section h2, div.section-hidden h2', function() {
+          toggle_visibility($(this));
+      });
+}
+function renderWarningMessageInLoginStatus(oauth, message) {
+  render_login_oauth(oauth, message)
 }
-
 
 function dispatcher_add(fun) {
     dispatcher_modules.push(fun);
@@ -54,32 +74,25 @@ function dispatcher_add(fun) {
 }
 
 function dispatcher() {
+    this.use('Title');
+    this.setTitle('RabbitMQ: ');
     for (var i in dispatcher_modules) {
         dispatcher_modules[i](this);
     }
 }
 
-function getParameterByName(name) {
-    var match = RegExp('[#&]' + name + '=([^&]*)').exec(window.location.hash);
-    return match && decodeURIComponent(match[1].replace(/\+/g, ' '));
-}
-
-function getAccessToken() {
-    return getParameterByName('access_token');
-}
-
 function start_app_login () {
   app = new Sammy.Application(function () {
     this.get('/', function () {})
     this.get('#/', function () {})
-    if (!oauth.enabled) {
+    if (!oauth.enabled || !oauth.oauth_disable_basic_auth) {
       this.put('#/login', function() {
         set_basic_auth(this.params['username'], this.params['password'])
         check_login()
       });
     }
   })
-  // TODO REFACTOR: this code can be simplified
+  
   if (oauth.enabled) {
     if (has_auth_credentials()) {
       check_login();
@@ -99,14 +112,13 @@ function check_login () {
   if (user == false || user.error) {
     clear_auth();
     if (oauth.enabled) {
-      hide_popup_warn();
-      renderWarningMessageInLoginStatus('Not authorized');
+      renderWarningMessageInLoginStatus(oauth, 'Not authorized');
     } else {
       replace_content('login-status', '<p>Login failed</p>');
     }
     return false;
   }
-
+  check_version()
   hide_popup_warn()
   replace_content('outer', format('layout', {}))
   var user_login_session_timeout = parseInt(user.login_session_timeout)
@@ -235,6 +247,7 @@ function dynamic_load(filename) {
     element.setAttribute('type', 'text/javascript');
     element.setAttribute('src', 'js/' + filename);
     document.getElementsByTagName('head')[0].appendChild(element);
+    return element;
 }
 
 function update_interval() {
@@ -262,7 +275,11 @@ function update_interval() {
 function go_to(url) {
     this.location = url;
 }
-
+function go_to_home() {
+    // location.href = rabbit_path_prefix() + "/"
+    location.href =  "/"
+  }
+  
 function set_timer_interval(interval) {
     timer_interval = interval;
     reset_timer();
@@ -277,6 +294,23 @@ function reset_timer() {
     }
 }
 
+function pause_auto_refresh() {
+    if (typeof globalThis.rmq_webui_auto_refresh_paused == 'undefined')
+        globalThis.rmq_webui_auto_refresh_paused = 0;
+
+    globalThis.rmq_webui_auto_refresh_paused++;
+    if (timer != null) {
+        clearInterval(timer);
+    }
+}
+
+function resume_auto_refresh() {
+    globalThis.rmq_webui_auto_refresh_paused--;
+    if (globalThis.rmq_webui_auto_refresh_paused == 0) {
+        reset_timer();
+    }
+}
+
 function update_manual(div, query) {
     var path;
     var template;
@@ -311,6 +345,7 @@ function update() {
     clearInterval(timer);
     with_update(function(html) {
             update_navigation();
+            update_warnings();
             replace_content('main', html);
             postprocess();
             postprocess_partial();
@@ -386,6 +421,41 @@ function update_navigation() {
     replace_content('rhs', l2);
 }
 
+function update_warnings() {
+    feature_flags = JSON.parse(sync_get('/feature-flags'));
+    var needs_enabling = false;
+    for (var i = 0; i < feature_flags.length; i++) {
+         var feature_flag = feature_flags[i];
+         if (feature_flag.state == "disabled" && feature_flag.stability != "experimental") {
+             needs_enabling = true;
+         }
+    }
+    deprecated_features = JSON.parse(sync_get('/deprecated-features/used'));
+    var needs_deprecating = false;
+    if (deprecated_features.length > 0) {
+        needs_deprecating = true;
+    }
+    var l1 = '<p class="warning">';
+    if (needs_enabling) {
+        l1 += '<span>&#9888;</span> All stable feature flags must be enabled after completing an upgrade. <a href="https://www.rabbitmq.com/feature-flags.html">[Learn more]</a>';
+    }
+    if (needs_deprecating) {
+        if (needs_enabling) {
+            l1 += '<br/>'
+        }
+        l1 += '<span>&#9888;</span> Deprecated features are being used. <a href="https://www.rabbitmq.com/feature-flags.html">[Learn more]</a>'
+    }
+    l1 += '</p>';
+    if (needs_enabling || needs_deprecating) {
+      $('#main').addClass('with-warnings');
+      $('#rhs').addClass('with-warnings');
+      replace_content('warnings', l1);
+  } else {
+      $('#main').removeClass('with-warnings');
+      $('#rhs').removeClass('with-warnings');
+  }
+}
+
 function navigation_tab_id(value) {
     return value.toLowerCase().replaceAll(/\s/g, "-")
 }
@@ -574,9 +644,13 @@ function show_popup(type, text, _mode) {
     hide();
     $('#outer').after(format('popup', {'type': type, 'text': text}));
     $(cssClass).fadeIn(100);
-    $(cssClass + ' span').on('click', function () {
+
+    var closeButtonCssClass = cssClass + ' span';
+    $('div#outer,' + closeButtonCssClass).on('click', function(event) {
+      if ($(event.target).eq($(closeButtonCssClass)) || !$(event.target).closest(cssClass).length) {
         $('.popup-owner').removeClass('popup-owner');
         hide();
+      }
     });
 }
 
@@ -632,6 +706,10 @@ function postprocess() {
                            "after deletion.");
         });
 
+    $('form.enable-feature-flag').on('submit', function() {
+                    full_refresh();
+    });
+
     $('label').map(function() {
             if ($(this).attr('for') == '') {
                 var id = 'auto-label-' + Math.floor(Math.random()*1000000000);
@@ -1238,7 +1316,8 @@ function update_status(status) {
 
 function with_req(method, path, body, fun) {
     if(!has_auth_credentials()) {
-        // navigate to the login form
+        // Clear any lingering auth settings in local storage and navigate to the login form.
+        clear_auth();
         location.reload();
         return;
     }
@@ -1271,13 +1350,14 @@ function get(url, accept, callback) {
   var req = new XMLHttpRequest();
   req.open("GET", url);
   req.setRequestHeader("Accept", accept);
-  req.send();
 
   req.onreadystatechange = function() {
     if (req.readyState == XMLHttpRequest.DONE) {
       callback(req);
     }
   };
+  req.send();
+
 }
 
 function sync_get(path) {
@@ -1339,16 +1419,16 @@ function sync_req(type, params0, path_template, options) {
         else
             // rabbitmq/rabbitmq-management#732
             // https://developer.mozilla.org/en-US/docs/Glossary/Truthy
-            return {result: true, http_status: req.status, req_params: params};
+            return {result: true, http_status: req.status, req_params: params, responseText: req.responseText};
     }
     else {
         return false;
     }
 }
-function initiate_logout(error = "") {
+function initiate_logout(oauth, error = "") {
     clear_pref('auth');
-    clear_cookie_value('auth');
-    renderWarningMessageInLoginStatus(error);
+    clear_cookie_value('auth');    
+    renderWarningMessageInLoginStatus(oauth, error);
 }
 function check_bad_response(req, full_page_404) {
     // 1223 == 204 - see https://www.enhanceie.com/ie/bugs.asp
@@ -1369,7 +1449,7 @@ function check_bad_response(req, full_page_404) {
 
         if (error == 'bad_request' || error == 'not_found' || error == 'not_authorised' || error == 'not_authorized') {
             if ((req.status == 401 || req.status == 403) && oauth.enabled) {
-              initiate_logout(reason);
+              initiate_logout(oauth, reason);
             } else {
               show_popup('warn', fmt_escape_html(reason));
             }
@@ -1680,40 +1760,20 @@ function select_queue_type(queuetype) {
     update();
 }
 
+function get_queue_type (queue) {    
+    return queue.type;
+}
+
 function is_quorum(queue) {
-    if (queue["arguments"]) {
-        if (queue["arguments"]["x-queue-type"]) {
-            return queue["arguments"]["x-queue-type"] === "quorum";
-        } else {
-            return false;
-        }
-    } else {
-        return false;
-    }
+    return get_queue_type(queue) === "quorum";
 }
 
 function is_stream(queue) {
-    if (queue["arguments"]) {
-        if (queue["arguments"]["x-queue-type"]) {
-            return queue["arguments"]["x-queue-type"] === "stream";
-        } else {
-            return false;
-        }
-    } else {
-        return false;
-    }
+    return get_queue_type(queue) === "stream";
 }
 
 function is_classic(queue) {
-    if (queue["arguments"]) {
-        if (queue["arguments"]["x-queue-type"]) {
-            return queue["arguments"]["x-queue-type"] === "classic";
-        } else {
-            return true;
-        }
-    } else {
-        return true;
-    }
+    return get_queue_type(queue) === "classic";
 }
 
 function ensure_queues_chart_range() {
@@ -1774,3 +1834,12 @@ function get_chart_range_type(arg) {
     console.log('[WARNING]: range type not found for arg: ' + arg);
     return 'basic';
 }
+
+function check_version() {
+    let curVersion = sync_get('/version')
+    let storedVersion = get_pref('version')
+    if (!storedVersion || storedVersion != curVersion) {
+        store_pref('version', curVersion)
+        location.reload()
+    }
+}
